from bitshares.amount import Amount
from bitshares.bitshares import BitShares
from bitshares.block import Block
from bitshares.account import Account
from bitshares.instance import set_shared_bitshares_instance
from websocket import create_connection
from datetime import datetime
import json
from bitshares.utils import formatTimeFromNow


bs = BitShares(
    node='wss://api.iamredbar.com/ws',
)
set_shared_bitshares_instance(bs)
ws = create_connection('wss://api.iamredbar.com/ws')
payload = {"id": 1,
           "method": "call",
           "params": [
               "history",
               "get_liquidity_pool_history",
               [
                   '1.19.0',
                   formatTimeFromNow(0),
                   None,
                   100,
                   63
               ]
           ]
           }
ws.send(json.dumps(payload))
result = ws.recv()
print(result)



# ACC_ID = '1.2.546734'
# ACC_NAME = 'iamredbar1'
# history_gen = Account(ACC_NAME).history()
# unknown_ops = []
#
# with open(f'{ACC_NAME}_readable_timestamp.txt', 'w') as file:
#     file.writelines('Parsed ops:\n')
#     for i in history_gen:
#         if i['op'][0] == 0:
#             if i['op'][1]['from'] == ACC_ID:
#                 file.writelines(f'Sent {Amount(i["op"][1]["amount"])} to {Account(i["op"][1]["to"]).name}. Time: {Block(i["block_num"])["timestamp"]}\n')
#             else:
#                 file.writelines(f'Received {Amount(i["op"][1]["amount"])} from {Account(i["op"][1]["from"]).name}. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 1:
#             file.writelines(f'Created order ref {i["result"][1]}: Sell {Amount(i["op"][1]["amount_to_sell"])} for min {Amount(i["op"][1]["min_to_receive"])}. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 2:
#             file.writelines(f'Cancelled order ref: {i["op"][1]["order"]}. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 4:
#             file.writelines(f'Order filled ref {i["op"][1]["order_id"]}: Sold {Amount(i["op"][1]["pays"])} for {Amount(i["op"][1]["receives"])}. Maker: {i["op"][1]["is_maker"]}. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 5:
#             file.writelines(f'Account <{i["op"][1]["name"]}> created. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 8:
#             file.writelines(f'Paid {Amount(i["op"][1]["fee"])} to upgrade to Lifetime Member. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 14:
#             file.writelines(f'{Amount(i["op"][1]["asset_to_issue"])} issued to {Account(i["op"][1]["issue_to_account"]).name}. Time: {Block(i["block_num"])["timestamp"]}\n')
#         elif i['op'][0] == 33:
#             file.writelines(f'Withdrew vesting balance of {Amount(i["op"][1]["amount"])}. Time: {Block(i["block_num"])["timestamp"]}\n')
#         # elif i['op'][0] == 38:
#         #     file.writelines(f'\n')
#         else:
#             unknown_ops.append(i)
#
#     file.writelines('Unparsed ops:\n')
#     for j in unknown_ops:
#         file.writelines(str(j) + '\n')



from pubsub import pub
from bitshares.bitshares import BitShares
from bitshares.asset import Asset
from bitshares.amount import Amount
from bitshares.market import Market
from bitshares.blockchain import Blockchain
from bitshares.instance import set_shared_blockchain_instance
from bitshares.utils import formatTimeFromNow
from pprint import pprint
import json
from websocket import create_connection

POOL_ID_LIMIT = 200


class Model:
    BLOCK_HOUR = 1200

    def __init__(self, **kwargs):
        self.bs = None
        self.keyed_bs = None
        self.ws = None
        self.node = None
        self.denomination = None
        self.pool_id = None
        self.asset_a_precision = None
        self.asset_b_precision = None
        self.share_asset_precision = None
        self.virtual_value = None
        self.invariant = None
        self.asset_a_id = None
        self.asset_a_symbol = None
        self.asset_a_balance = None
        self.asset_b_id = None
        self.asset_b_symbol = None
        self.asset_b_balance = None
        self.share_asset_id = None
        self.share_asset_symbol = None
        self.share_asset_balance = None
        self.swap_fee = None
        self.withdraw_fee = None
        self.price_switch = 'b/a'
        self.history = []
        self.usd_val_a = 0
        self.usd_val_b = 0
        self.bts_val_a = 0
        self.bts_val_b = 0
        self.cny_val_a = 0
        self.cny_val_b = 0

    def pool_change(self, new_pool_str):
        new_data = {}
        pool_obj = self.bs.rpc.get_object(new_pool_str.split(' ')[1])
        self.usd_val_a = 0
        self.usd_val_b = 0
        self.bts_val_a = 0
        self.bts_val_b = 0
        self.cny_val_a = 0
        self.cny_val_b = 0
        asset_a = Asset(pool_obj['asset_a'])
        asset_b = Asset(pool_obj['asset_b'])
        share_asset = Asset(pool_obj['share_asset'])
        self.pool_id = pool_obj['id']
        self.asset_a_precision = asset_a.precision
        new_data['asset_a_precision'] = self.asset_a_precision
        self.asset_b_precision = asset_b.precision
        new_data['asset_b_precision'] = self.asset_b_precision
        self.share_asset_precision = share_asset.precision
        self.virtual_value = pool_obj['virtual_value']
        self.invariant = int(self.virtual_value) / (10 ** (int(self.asset_a_precision) + int(self.asset_b_precision)))
        self.asset_a_id = asset_a.identifier
        new_data['asset_a_id'] = self.asset_a_id
        self.asset_a_symbol = asset_a.symbol
        new_data['asset_a_symbol'] = self.asset_a_symbol
        self.asset_a_balance = str(int(pool_obj['balance_a']) / 10 ** asset_a.precision)
        new_data['asset_a_balance'] = self.asset_a_balance
        self.asset_b_id = asset_b.identifier
        new_data['asset_b_id'] = self.asset_b_id
        self.asset_b_symbol = asset_b.symbol
        new_data['asset_b_symbol'] = self.asset_b_symbol
        self.asset_b_balance = str(int(pool_obj['balance_b']) / 10 ** asset_b.precision)
        new_data['asset_b_balance'] = self.asset_b_balance
        self.share_asset_id = share_asset.identifier
        new_data['share_asset_id'] = self.share_asset_id
        self.share_asset_symbol = share_asset.symbol
        new_data['share_asset_symbol'] = self.share_asset_symbol
        temp = self.bs.rpc.get_object(share_asset['dynamic_asset_data_id'])['current_supply']
        self.share_asset_balance = str(int(temp) / (10 ** share_asset['precision']))
        new_data['share_asset_balance'] = self.share_asset_balance
        Asset.clear_cache()
        # this is the generation of the stats panel information
        self.swap_fee = f'{pool_obj["taker_fee_percent"] / 100}%'
        new_data['swap_fee'] = self.swap_fee
        self.withdraw_fee = f'{pool_obj["withdrawal_fee_percent"] / 100}%'
        new_data['withdraw_fee'] = self.withdraw_fee
        history_info = self.generate_history(new_pool_str.split(' ')[1])
        # Get pool value
        self._get_pool_value(
            self.asset_a_symbol,
            self.asset_a_balance,
            self.asset_b_symbol,
            self.asset_b_balance,
        )
        if self.denomination == 'BitUSD':
            _precision = 2
            new_data['value'] = f'{(self.usd_val_a + self.usd_val_b):.2f} USD'
        elif self.denomination == 'BTS':
            _precision = 5
            new_data['value'] = f'{(self.bts_val_a + self.bts_val_b):.5f} BTS'
        elif self.denomination == 'BitCNY':
            _precision = 4
            new_data['value'] = f'{(self.cny_val_a + self.cny_val_b):.4f} CNY'
        new_data['swap_count'] = history_info['swap_count']
        new_data['history'] = []
        for item in history_info['history']:
            new_data['history'].append(item)
        if float(self.share_asset_balance) != 0:
            new_data['poolshare_value'] = f'{round(float(new_data["value"].split(" ")[0]) / float(self.share_asset_balance), _precision)} {new_data["value"].split(" ")[1]}'
        else:
            new_data['poolshare_value'] = 0
        pub.sendMessage('update_pool_change', data=new_data)

    def display_pool_value(self):
        pass

    def price_swap(self):
        self.price_switch = 'a/b' if self.price_switch == 'b/a' else 'b/a'
        history = []
        for op in self.history:
            if self.price_switch == 'a/b':
                history.append(
                    {
                        'asset_a': op['asset_a'],
                        'icon': op['icon'],
                        'asset_b': op['asset_b'],
                        'price': op['price_ab'],
                        'price_color': op['price_ab_color'],
                    }
                )
            else:
                history.append(
                    {
                        'asset_a': op['asset_a'],
                        'icon': op['icon'],
                        'asset_b': op['asset_b'],
                        'price': op['price_ba'],
                        'price_color': op['price_ba_color'],
                    }
                )
        pub.sendMessage('refresh_history_panel', data=history)

    def generate_history(self, pool):
        return_data = {}
        op_list = {}
        payload = {"id": 1,
                   "method": "call",
                   "params": [
                       "history",
                       "get_liquidity_pool_history",
                       [
                           pool,
                           formatTimeFromNow(0),
                           None,
                           100,
                           63
                       ]
                   ]
                   }
        self._set_ws_connection()
        self.ws.send(json.dumps(payload))
        result = self.ws.recv()
        r = json.loads(result)
        if r['result']:
            for i in r['result']:
                if not i['sequence'] in op_list.keys():
                    op_list[i['sequence']] = i
        sort_order = sorted(op_list, reverse=True)
        return_data['history'] = []
        _prev_price = None
        for index, op in enumerate(sort_order):
            # the dictionary keys are always asset_a, icon, asset_b, price, price_color
            # this determines what is for sale, and puts the correct arrow and asset order
            if op_list[op]['op']['op'][1]['amount_to_sell']['asset_id'] == self.asset_a_id:
                icon = 'arrow-right'
                asset_a = str(Amount(op_list[op]['op']['result'][1]['paid'][0]))
                asset_b = str(Amount(op_list[op]['op']['result'][1]['received'][0]))
            else:
                icon = 'arrow-left'
                asset_a = str(Amount(op_list[op]['op']['result'][1]['received'][0]))
                asset_b = str(Amount(op_list[op]['op']['result'][1]['paid'][0]))
            try:
                if op_list[sort_order[index + 1]]['op']['op'][1]['amount_to_sell']['asset_id'] == self.asset_a_id:
                    _asset_a = str(Amount(op_list[sort_order[index + 1]]['op']['result'][1]['paid'][0]))
                    _asset_b = str(Amount(op_list[sort_order[index + 1]]['op']['result'][1]['received'][0]))
                else:
                    _asset_a = str(Amount(op_list[sort_order[index + 1]]['op']['result'][1]['received'][0]))
                    _asset_b = str(Amount(op_list[sort_order[index + 1]]['op']['result'][1]['paid'][0]))
            except:
                pass
            _price_ab = float(asset_a.split(' ')[0].replace(',', '')) / float(asset_b.split(' ')[0].replace(',', ''))
            _price_ba = float(asset_b.split(' ')[0].replace(',', '')) / float(asset_a.split(' ')[0].replace(',', ''))
            # check previous y/x price, change color depending on result
            price_ab_color = '#808080'
            price_ba_color = '#808080'
            if index != 99:
                _prev_price_ab = float(_asset_a.split(' ')[0].replace(',', '')) / float(_asset_b.split(' ')[0].replace(',', ''))
                _prev_price_ba = float(_asset_b.split(' ')[0].replace(',', '')) / float(_asset_a.split(' ')[0].replace(',', ''))
                if _price_ab > _prev_price_ab:
                    price_ab_color = '#669f38'
                    price_ba_color = '#e9002c'
                elif _price_ab < _prev_price_ab:
                    price_ab_color = '#e9002c'
                    price_ba_color = '#669f38'
            self.history.append(
                {
                    'asset_a': asset_a,
                    'icon': icon,
                    'asset_b': asset_b,
                    'price_ab': f'{_price_ab:.3f}',
                    'price_ab_color': price_ab_color,
                    'price_ba': f'{_price_ba:.3f}',
                    'price_ba_color': price_ba_color,
                }
            )
        # Generate activity from history for stats panel
        _last_confirmed_block = Blockchain(mode='irreversible').get_current_block_num()
        swap_count = 0
        for op in sort_order:
            if op_list[op]['op']['block_num'] > _last_confirmed_block - (self.BLOCK_HOUR * 24):
                swap_count += 1
        return_data['swap_count'] = swap_count
        for op in self.history:
            if self.price_switch == 'b/a':
                return_data['history'].append(
                    {
                        'asset_a': op['asset_a'],
                        'icon': op['icon'],
                        'asset_b': op['asset_b'],
                        'price': op['price_ba'],
                        'price_color': op['price_ba_color'],
                    }
                )
            else:
                return_data['history'].append(
                    {
                        'asset_a': op['asset_a'],
                        'icon': op['icon'],
                        'asset_b': op['asset_b'],
                        'price': op['price_ab'],
                        'price_color': op['price_ab_color'],
                    }
                )
        return return_data

    def set_denomination(self, data):
        self.denomination = data
        self._get_pool_value(
            self.asset_a_symbol,
            self.asset_a_balance,
            self.asset_b_symbol,
            self.asset_b_balance,
        )
        new_data = {}
        if self.denomination == 'BitUSD':
            _precision = 2
            new_data['value'] = f'{(self.usd_val_a + self.usd_val_b):.2f} USD'
        elif self.denomination == 'BTS':
            _precision = 5
            new_data['value'] = f'{(self.bts_val_a + self.bts_val_b):.5f} BTS'
        elif self.denomination == 'BitCNY':
            _precision = 4
            new_data['value'] = f'{(self.cny_val_a + self.cny_val_b):.4f} CNY'
        if float(self.share_asset_balance) != 0:
            new_data['poolshare_value'] = f'{round(float(new_data["value"].split(" ")[0]) / float(self.share_asset_balance), _precision)} {new_data["value"].split(" ")[1]}'
        else:
            new_data['poolshare_value'] = 0
        pub.sendMessage('refresh_stats_panel', data=new_data)

    def correct_pool_amount(self, higher_amount, lower_amount, asset):
        precision = Asset(asset).precision
        Asset.clear_cache()
        difference = higher_amount - lower_amount
        return float(round(difference, precision))

    # exchange preserves x*y
    # buying subtracts from that side of the pool
    # selling adds to that side of the pool
    def update_pool_price(self, data):
        if data['swap_asset'] == self.asset_a_symbol:
            swap_flag = 'a'
            reference = float(self.asset_a_balance)
        else:
            swap_flag = 'b'
            reference = float(self.asset_b_balance)
        addition_to = reference + float(data['amount'])
        new_balance = self.invariant / addition_to
        output_amount = self.correct_pool_amount(
            float(self.asset_b_balance) if swap_flag == 'a' else float(self.asset_a_balance),
            new_balance,
            data['purchase_asset']
        )
        pub.sendMessage('update_pool_estimate', data=str(output_amount))

    def _set_ws_connection(self):
        self.ws = create_connection(self.node)

    def get_pools(self, data):
        self.node = data['node']
        self.denomination = data['denomination']
        self.bs = BitShares(
            node=self.node,
            nobroadcast=True,
            blocking='head',
        )
        set_shared_blockchain_instance(self.bs)
        self._set_ws_connection()
        _counter = 0
        data = []
        while _counter < POOL_ID_LIMIT:
            payload_string = "1.19." + str(_counter)
            payload1 = {
                "id": 1,
                "method": "call",
                "params": [
                    "database",
                    "list_liquidity_pools",
                    [
                        100,
                        payload_string,
                        False
                    ]
                ]
            }
            self.ws.send(json.dumps(payload1))
            result1 = self.ws.recv()
            r = json.loads(result1)
            # print(r)
            for i in r['result']:
                data.append(f'{Asset(self.bs.rpc.get_object(i["id"])["share_asset"]).symbol} {i["id"]}')
            _counter += 100
        pub.sendMessage('return_pool_list', data=data)

    def _get_pool_value(self, asset_a, amount_a, asset_b, amount_b):
        # check each asset separately, first for USD, then BTS, then derive
        # asset a
        if asset_a == 'USD':
            self.usd_val_a = float(amount_a)
        if asset_a == 'BTS':
            self.bts_val_a = float(amount_a)
        if asset_a == 'CNY':
            self.cny_val_a = float(amount_a)
        # derive BTS then others, validate they aren't already there
        if self.bts_val_a == 0:
            self.bts_val_a = round(float(str(Market(f'{asset_a}/BTS').ticker()['latest']).split(' ')[0]) * float(amount_a), 5)
        if self.usd_val_a == 0:
            bts_usd_val = float(str(Market('BTS/USD').ticker()['latest']).split(' ')[0])
            self.usd_val_a = self.bts_val_a * bts_usd_val
        if self.cny_val_a == 0:
            bts_cny_val = float(str(Market('BTS/CNY').ticker()['latest']).split(' ')[0])
            self.cny_val_a = self.bts_val_a * bts_cny_val
        # asset b
        if asset_b == 'USD':
            self.usd_val_b = float(amount_b)
        if asset_b == 'BTS':
            self.bts_val_b = float(amount_b)
        if asset_b == 'CNY':
            self.cny_val_b = float(amount_b)
        # derive
        if self.bts_val_b == 0:
            self.bts_val_b = round(float(str(Market(f'{asset_b}/BTS').ticker()['latest']).split(' ')[0]) * float(amount_b), 5)
        if self.usd_val_b == 0:
            bts_usd_val = float(str(Market('BTS/USD').ticker()['latest']).split(' ')[0])
            self.usd_val_b = self.bts_val_b * bts_usd_val
        if self.cny_val_b == 0:
            bts_cny_val = float(str(Market('BTS/CNY').ticker()['latest']).split(' ')[0])
            self.cny_val_b = self.bts_val_b * bts_cny_val

    # def _verify_active_key(self, key):
    #     """
    #     returns account from private key
    #     this will be used to verify whether or not the
    #     supplied key will be sufficient enough to do the
    #     operation
    #     """
    #     print(self.bs.wallet.getAccountFromPrivateKey(key))

    def _set_keyed_bs(self, key):
        self.keyed_bs = BitShares(
            node=self.node,
            keys=key,
            blocking='head',
            nobroadcast=False,
        )

    def swap_assets(self, data):
        self._set_keyed_bs(data['key'])
        amount_to_sell = Amount(data["sell_amount"], data["sell_asset"])
        anticipated = Amount(data['receive_approx_amount'], data['receive_asset'])
        min_to_receive = anticipated * 0.993
        trade_message = self.keyed_bs.exchange_with_liquidity_pool(
            pool=self.pool_id,
            amount_to_sell=amount_to_sell,
            min_to_receive=min_to_receive,
            account=data['account'],
        )
        return_data = {
            'anticipated': anticipated,
            'paid': Amount(trade_message['operation_results'][0][1]['paid'][0]),
            'received': Amount(trade_message['operation_results'][0][1]['received'][0]),
            'operation_results': trade_message['operation_results'],
            'interaction_type': 'swap',
        }
        pub.sendMessage('interaction_return', data=return_data)

    def deposit_assets(self, data):
        self._set_keyed_bs(data['key'])
        trade_message = self.keyed_bs.deposit_into_liquidity_pool(
            pool=self.pool_id,
            amount_a=Amount(data['amount_a'], data['asset_a']),
            amount_b=Amount(data['amount_b'], data['asset_b']),
            account=data['account'],
        )
        return_data = {
            'paid_a': Amount(trade_message['operation_results'][0][1]['paid'][0]),
            'paid_b': Amount(trade_message['operation_results'][0][1]['paid'][1]),
            'received': Amount(trade_message['operation_results'][0][1]['received'][0]),
            'operation_results': trade_message['operation_results'],
            'interaction_type': 'deposit',
        }
        pub.sendMessage('interaction_return', data=return_data)

    def withdraw_assets(self, data):
        self._set_keyed_bs(data['key'])
        trade_message = self.keyed_bs.withdraw_from_liquidity_pool(
            pool=self.pool_id,
            share_amount=Amount(data['amount'], data['share_asset']),
            account=data['account'],
        )
        return_data = {
            'exchanged': Amount(trade_message['operation_results'][0][1]['paid'][0]),
            'received_a': Amount(trade_message['operation_results'][0][1]['received'][0]),
            'received_b': Amount(trade_message['operation_results'][0][1]['received'][1]),
            'operation_results': trade_message['operation_results'],
            'interaction_type': 'withdraw',
        }
        pub.sendMessage('interaction_return', data=return_data)



from PoolTool.view import PoolTool
from PoolTool.model import Model
from pubsub import pub


class Controller:
    def __init__(self):
        # initiate model and view so that controller can talk
        self.model = Model()
        self.view = PoolTool()
        # subscribe and respond to actions
        pub.subscribe(self.pool_change, 'pool_change')
        pub.subscribe(self.update_pool_change, 'update_pool_change')
        pub.subscribe(self.update_pool_price, 'update_pool_price')
        pub.subscribe(self.update_pool_estimate, 'update_pool_estimate')
        pub.subscribe(self.get_pools, 'get_pools')
        pub.subscribe(self.return_pool_list, 'return_pool_list')
        pub.subscribe(self.swap_assets, 'swap_assets')
        pub.subscribe(self.deposit_assets, 'deposit_assets')
        pub.subscribe(self.withdraw_assets, 'withdraw_assets')
        pub.subscribe(self.interaction_return, 'interaction_return')
        pub.subscribe(self.set_denomination, 'set_denomination')
        pub.subscribe(self.price_swap, 'price_swap')
        pub.subscribe(self.refresh_history_panel, 'refresh_history_panel')
        pub.subscribe(self.refresh_stats_panel, 'refresh_stats_panel')

    def pool_change(self, data):
        self.model.pool_change(data)

    def update_pool_change(self, data):
        self.view.update_pool_change(data)

    def update_pool_price(self, data):
        self.model.update_pool_price(data)

    def update_pool_estimate(self, data):
        self.view.update_pool_estimate(data)

    def get_pools(self, data):
        self.model.get_pools(data)

    def return_pool_list(self, data):
        self.view.return_pool_list(data)

    def swap_assets(self, data):
        self.model.swap_assets(data)

    def deposit_assets(self, data):
        self.model.deposit_assets(data)

    def withdraw_assets(self, data):
        self.model.withdraw_assets(data)

    def interaction_return(self, data):
        self.view.interaction_return(data)

    def set_denomination(self, data):
        self.model.set_denomination(data)

    def price_swap(self):
        self.model.price_swap()

    def refresh_history_panel(self, data):
        self.view.generate_history_panel(data)

    def refresh_stats_panel(self, data):
        self.view.generate_stats_panel(data)


def main():
    c = Controller()
    c.view.run()

if __name__ == '__main__':
    main()


import platform, pathlib
from kivymd.app import MDApp
from kivymd.theming import ThemableBehavior
from kivymd.uix.behaviors import RectangularElevationBehavior
from kivymd.uix.boxlayout import MDBoxLayout
from kivymd.uix.gridlayout import MDGridLayout
from kivy.lang import Builder
from kivy import Config
from kivy.core.window import Window
from kivy.properties import StringProperty, ColorProperty
from kivy.uix.popup import Popup
from kivy.clock import Clock
from pubsub import pub

HERE = pathlib.Path(__file__).parent
Window.size = (400, 650)

# Fix bug where Windows does not see OpenGL 2.0
if platform.system() == 'Windows':
    import os
    Config.set('graphics', 'multisamples', '0')
    os.environ['KIVY_GL_BACKEND'] = 'angle_sdl2'

class CustomToolbar(ThemableBehavior, RectangularElevationBehavior, MDBoxLayout):
    pass


class CustomListItem(MDGridLayout):
    asset_a = StringProperty()
    icon = StringProperty()
    asset_b = StringProperty()
    price = StringProperty()
    price_color = ColorProperty()


class LoadingPoolsPopup(Popup):
    pass


class InteractPopup(Popup):
    titletext = StringProperty()
    bodytext = StringProperty()
    confirmbutton = StringProperty()
    data = None


class ReturnPopup(Popup):
    titletext = StringProperty()
    bodytext = StringProperty()


class SettingsPopup(Popup):
    account = StringProperty()
    key = StringProperty()
    node = StringProperty()
    denomination = StringProperty()


class LoadingPoolInfo(Popup):
    bodytext = StringProperty()


class PoolTool(MDApp):
    """

    """

    def __init__(self, **kwargs):
        self.icon = f'{HERE}/assets/icon.jpg'
        super().__init__(**kwargs)
        self.view_pools = []
        self.screen = Builder.load_file(f'{HERE}/layout.kv')
        self.pool_dropdown = []
        self.interact_popup = None
        self.interact_data = None
        self.settings_popup = None
        self.loading_pools_popup = None
        self.loading_pool_info = None
        self.account = ''
        self.key = ''
        self.node = 'wss://api.iamredbar.com/ws'  # default in settings
        self.denomination = 'BitUSD'  # default in settings
        self.screen.pool_select_spinner.text = 'Select Pool Here'
        self._init_gui()

    def _init_gui(self):
        self.screen.amount_text_field.text = ''
        self.screen.swap_asset_button.text = 'Asset A'
        self.screen.swap_estimate.text = 'Receive (estimate)'
        self.screen.other_asset.text = 'Asset B'
        self.screen.deposit_a_text_field.text = ''
        self.screen.deposit_asset_a.text = 'Asset A'
        self.screen.deposit_b_text_field.text = ''
        self.screen.deposit_asset_b.text = 'Asset B'
        self.screen.withdraw_text_field.text = ''
        self.screen.withdraw_share_asset_label.text = 'Share Asset'
        self.screen.pool_share_balance.text = '(Share Balance)'
        self.screen.pool_share_bal_label.text = 'Share Asset'

    def _clear_gui(self):
        self.screen.amount_text_field.text = ''
        self.screen.swap_asset_button.text = ''
        self.screen.swap_estimate.text = 'Receive (estimate)'
        self.screen.other_asset.text = ''
        self.screen.deposit_a_text_field.text = ''
        self.screen.deposit_asset_a.text = ''
        self.screen.deposit_b_text_field.text = ''
        self.screen.deposit_asset_b.text = ''
        self.screen.withdraw_text_field.text = ''
        self.screen.withdraw_share_asset_label.text = ''
        self.screen.pool_share_balance.text = ''
        self.screen.pool_share_bal_label.text = ''
        self.screen.history_list.clear_widgets()

    def get_pools(self, caller):
        if not self.screen.pool_select_spinner.values or caller != 'spinner':
            if self.loading_pools_popup:
                self.loading_pools_popup = None
            if not self.loading_pools_popup:
                self.loading_pools_popup = LoadingPoolsPopup()
            self.loading_pools_popup.open()
            data = {
                'node': self.node,
                'denomination': self.denomination,
            }
            Clock.schedule_once(lambda dt: pub.sendMessage('get_pools', data=data), 0.2)

    def pool_change(self, new_pool):
        if new_pool != 'Select Pool Here':
            if self.loading_pool_info:
                self.loading_pool_info = None
            if not self.loading_pool_info:
                temp = f'loading\n{new_pool.split(" ")[0]}\ninfo...'
                self.loading_pool_info = LoadingPoolInfo(bodytext=temp)
            self.loading_pool_info.open()
            Clock.schedule_once(lambda dt: pub.sendMessage(topicName='pool_change', data=new_pool))

    def refresh_stats_panel(self, data):
        pass

    def return_pool_list(self, data):
        self._clear_popups()
        self.screen.pool_select_spinner.values = data

    def build(self):
        self.theme_cls.theme_style = 'Dark'
        self.theme_cls.primary_palette = 'LightGreen'
        self.theme_cls.accent_palette = 'Red'
        self.use_kivy_settings = False
        Config.set('input', 'mouse', 'mouse,multitouch_on_demand')
        return self.screen

    """
    These are functions that are triggered by the GUI
    """
    def _open_settings(self):
        if self.settings_popup:
            self.settings_popup = None
        if not self.settings_popup:
            self.settings_popup = SettingsPopup(
                account=self.account,
                key=self.key,
                node=self.node,
                denomination=self.denomination,
            )
        self.settings_popup.open()

    def save_settings(self, account, key, node, denomination):
        self.account = account
        self.key = key
        self.node = node
        self.denomination = denomination
        pub.sendMessage('set_denomination', data=denomination)
        self._clear_popups()

    def _clear_popups(self):
        if self.settings_popup:
            self.settings_popup.dismiss(force=True)
            self.settings_popup = None
        if self.loading_pools_popup:
            self.loading_pools_popup.dismiss(force=True)
            self.loading_pools_popup = None
        if self.interact_popup:
            self.interact_popup.dismiss(force=True)
            self.interact_popup = None
            self.interact_data = None
        if self.loading_pool_info:
            self.loading_pool_info.dismiss(force=True)
            self.loading_pool_info = None

    def update_pool_change(self, data):
        self._clear_gui()
        self._clear_popups()
        self.screen.swap_card.disabled = False
        self.screen.deposit_card.disabled = False
        self.screen.withdraw_card.disabled = False
        self.screen.swap_asset_button.text = data['asset_a_symbol']
        self.screen.other_asset.text = data['asset_b_symbol']
        self.screen.deposit_asset_a.text = data['asset_a_symbol']
        self.screen.deposit_asset_b.text = data['asset_b_symbol']
        self.screen.pool_share_balance.text = data['share_asset_balance']
        self.screen.pool_share_bal_label.text = data['share_asset_symbol']
        self.screen.withdraw_share_asset_label.text = data['share_asset_symbol']
        self.screen.value_label.text = f'{data["value"]}'
        self.screen.swap_count.text = str(data['swap_count']) if data['swap_count'] < 100 else '100+'
        self.screen.swap_fee.text = data['swap_fee']
        self.screen.withdraw_fee.text = data['withdraw_fee']
        self.screen.poolshare_value.text = f'{data["poolshare_value"]}\nper share'
        self.screen.asset_a_balance.text = f'{data["asset_a_balance"]}\n{data["asset_a_symbol"]}'
        self.screen.asset_b_balance.text = f'{data["asset_b_balance"]}\n{data["asset_b_symbol"]}'
        self.generate_history_panel(data['history'])

    def generate_history_panel(self, history):
        # print(self.screen.history_list.children)
        # self.screen.history_list.clear_widgets()
        # try:
        #     for child in self.screen.history_list.children:
        #         print(child)
        #         self.screen.history_list.remove_widget(child)
        # except:
        #     print('no children')
        for op in history:
            self.screen.history_list.add_widget(
                CustomListItem(
                    asset_a=op['asset_a'],
                    icon=op['icon'],
                    price=op['price'],
                    price_color=op['price_color'],
                    asset_b=op['asset_b'],
                )
            )
        # print(self.screen.history_list.children)

    def generate_stats_panel(self, data):
        self.screen.value_label.text = f'{data["value"]}'
        self.screen.poolshare_value.text = f'{data["poolshare_value"]}\nper share'

    def _interchange_assets_swap(self):
        tmp_a = self.screen.swap_asset_button.text
        tmp_b = self.screen.other_asset.text
        self.screen.swap_asset_button.text = tmp_b
        self.screen.other_asset.text = tmp_a
        self.screen.amount_text_field.text = ''
        self.screen.swap_button.disabled = True
        self.screen.swap_estimate.text = 'Receive (estimate)'

    def update_pool_price(self, amount):
        send_data = {
            'amount': amount,
            'swap_asset': self.screen.swap_asset_button.text,
            'purchase_asset': self.screen.other_asset.text
        }
        pub.sendMessage('update_pool_price', data=send_data)

    def price_swap(self, sender):
        if sender == 'price_switch':
            pub.sendMessage('price_swap')

    def update_pool_estimate(self, amount):
        self.screen.swap_estimate.text = amount
        self.screen.swap_button.disabled = False

    def _swap_assets(self, data):
        pub.sendMessage('swap_assets', data=data)
        self._clear_popups()

    def _deposit_assets(self, data):
        pub.sendMessage('deposit_assets', data=data)
        self._clear_popups()

    def _withdraw_assets(self, data):
        pub.sendMessage('withdraw_assets', data=data)
        self._clear_popups()

    def interact(self, btn_text):
        if btn_text == 'SWAP':
            data = {
                'sell_asset': self.screen.swap_asset_button.text,
                'sell_amount': self.screen.amount_text_field.text,
                'receive_asset': self.screen.other_asset.text,
                'receive_approx_amount': self.screen.swap_estimate.text,
            }
            titletext = 'Verify Swap'
            bodytext = f'Swap\n{data["sell_amount"]} {data["sell_asset"]}\nfor approximately\n{data["receive_approx_amount"]} {data["receive_asset"]}?'
            confirmbutton = 'SWAP'
        elif btn_text == 'DEPOSIT':
            data = {
                'asset_a': self.screen.deposit_asset_a.text,
                'amount_a': self.screen.deposit_a_text_field.text,
                'asset_b': self.screen.deposit_asset_b.text,
                'amount_b': self.screen.deposit_b_text_field.text,
            }
            titletext = 'Verify Deposit'
            bodytext = f'Deposit:\n{data["amount_a"]} {data["asset_a"]}\n{data["amount_b"]} {data["asset_b"]}\nfor\n{self.screen.withdraw_share_asset_label.text}?'
            confirmbutton = 'DEPOSIT'
        elif btn_text == 'WITHDRAW':
            data = {
                'share_asset': self.screen.withdraw_share_asset_label.text,
                'amount': self.screen.withdraw_text_field.text,
                'asset_a': self.screen.deposit_asset_a.text,
                'asset_b': self.screen.deposit_asset_b.text,
            }
            titletext = 'Verify Withdraw'
            bodytext = f'Withdraw:\n{data["amount"]} {data["share_asset"]}\nfor\n{data["asset_a"]} and {data["asset_b"]}?'
            confirmbutton = 'WITHDRAW'
        else:
            data = {}
            titletext = ''
            bodytext = ''
            confirmbutton = ''
        self.interact_data = data
        if self.interact_popup:
            self.interact_popup = None
        if not self.interact_popup:
            self.interact_popup = InteractPopup(
                titletext=titletext,
                bodytext=bodytext,
                confirmbutton=confirmbutton,
            )
        self.interact_popup.open()

    def _confirm_interaction(self, interaction):
        # verify that account and key have been specified
        if self.account and self.key:
            self.interact_data['account'] = self.account
            self.interact_data['key'] = self.key
            if interaction == 'SWAP':
                self._swap_assets(self.interact_data)
            elif interaction == 'DEPOSIT':
                self._deposit_assets(self.interact_data)
            elif interaction == 'WITHDRAW':
                self._withdraw_assets(self.interact_data)
        else:  # they haven't entered an account or key
            self._clear_popups()
            popup = ReturnPopup(
                titletext='Invalid Account Settings',
                bodytext='Please check account\ninformation in\nthe settings',
            )
            popup.open()

    def interaction_return(self, data):
        if data['interaction_type'] == 'swap':
            titletext = 'Swap Complete'
            bodytext = f'Paid {data["paid"]}\nfor {data["received"]}\n(anticipated {data["anticipated"]})'
        elif data['interaction_type'] == 'deposit':
            titletext = 'Deposit Complete'
            bodytext = f'Deposited {data["paid_a"]}\nand {data["paid_b"]}\nfor {data["received"]}'
        elif data['interaction_type'] == 'withdraw':
            titletext = 'Withdraw Complete'
            bodytext = f'Exchanged {data["exchanged"]}\nfor {data["received_a"]}\nand {data["received_b"]}'
        else:
            titletext = 'Oops'
            bodytext = "You shouldn't ever see this..."
        popup = ReturnPopup(
            titletext=titletext,
            bodytext=bodytext,
        )
        popup.open()



import pathlib, platform
from setuptools import find_packages, setup


HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

if platform.system() == 'Windows':
    I_R = [
        'cython==0.29.22',
        'bitshares',
        'pypubsub',
        'kivy[base]',
        'kivymd'
    ]
else:
    I_R = [
        'cython',
        'bitshares',
        'pypubsub',
        'kivy',
        'kivymd'
    ]

setup(
    name='PoolTool',
    version='2.1.2',
    description='A simple Python tool to help anyone use Liquidity Pools on the BitShares blockchain.',
    long_description=README,
    long_description_content_type='text/markdown',
    author='Brendan Jensen (iamredbar)',
    author_email='iamredbar@protonmail.com',
    license='MIT',
    url='https://github.com/iamredbar/PoolTool',
    packages=find_packages(),
    install_requires=I_R,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    package_data={
        'PoolTool': ['*.kv', 'assets/*.jpg'],
    },
    entry_points={
        'console_scripts': [
            'PoolTool = PoolTool.controller:main'
        ]
    }
)
